@@ -2,38 +2,42 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
 **
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Contact: Nokia Corporation (info@qt.nokia.com)
 **
-** Commercial Usage
-**
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
 **
 ** GNU Lesser General Public License Usage
 **
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this file.
+** Please review the following information to ensure the GNU Lesser General
+** Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** If you are unsure which license is appropriate for your use, please
-** contact the sales department at http://qt.nokia.com/contact.
+** In addition, as a special exception, Nokia gives you certain additional
+** rights. These rights are described in the Nokia Qt LGPL Exception
+** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+**
+** Other Usage
+**
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+** If you have questions regarding the use of this file, please contact
+** Nokia at qt-info@nokia.com.
 **
 **************************************************************************/
 
 #include "pluginspec.h"
-#include "pluginspec.h"
+
 #include "pluginspec_p.h"
 #include "iplugin.h"
 #include "iplugin_p.h"
 #include "pluginmanager.h"
 
+#include <QtCore/QDir>
 #include <QtCore/QFile>
 #include <QtCore/QFileInfo>
 #include <QtCore/QXmlStreamReader>
@@ -82,6 +86,23 @@
     Version string that a plugin must match to fill this dependency.
 */
 
+/*!
+    \variable ExtensionSystem::PluginDependency::type
+    Defines whether the dependency is required or optional.
+    \sa ExtensionSystem::PluginDependency::Type
+*/
+
+/*!
+    \enum ExtensionSystem::PluginDependency::Type
+    Whether the dependency is required or optional.
+    \value Required
+           Dependency needs to be there.
+    \value Optional
+           Dependency is not necessarily needed. You need to make sure that
+           the plugin is able to load without this dependency installed, so
+           for example you may not link to the dependency's library.
+*/
+
 /*!
     \class ExtensionSystem::PluginSpec
     \brief Contains the information of the plugins xml description file and
@@ -118,20 +139,30 @@
             extensionsInitialized has been called. The loading process is
             complete.
     \value Stopped
-            The plugin has been shut down, i.e. the plugin's IPlugin::shutdown() method has been called.
+            The plugin has been shut down, i.e. the plugin's IPlugin::aboutToShutdown() method has been called.
     \value Deleted
             The plugin instance has been deleted.
 */
+
 using namespace ExtensionSystem;
 using namespace ExtensionSystem::Internal;
 
+/*!
+    \fn uint qHash(const ExtensionSystem::PluginDependency &value)
+    \internal
+*/
+uint ExtensionSystem::qHash(const ExtensionSystem::PluginDependency &value)
+{
+    return qHash(value.name);
+}
+
 /*!
     \fn bool PluginDependency::operator==(const PluginDependency &other)
     \internal
 */
-bool PluginDependency::operator==(const PluginDependency &other)
+bool PluginDependency::operator==(const PluginDependency &other) const
 {
-    return name == other.name && version == other.version;
+    return name == other.name && version == other.version && type == other.type;
 }
 
 /*!
@@ -225,6 +256,44 @@ QString PluginSpec::url() const
     return d->url;
 }
 
+/*!
+    \fn QString PluginSpec::category() const
+    The category that the plugin belongs to. Categories are groups of plugins which allow for keeping them together in the UI.
+    Returns an empty string if the plugin does not belong to a category.
+*/
+QString PluginSpec::category() const
+{
+    return d->category;
+}
+
+/*!
+    \fn bool PluginSpec::isExperimental() const
+    Returns if the plugin has its experimental flag set.
+*/
+bool PluginSpec::isExperimental() const
+{
+    return d->experimental;
+}
+
+/*!
+    \fn bool PluginSpec::isEnabled() const
+    Returns if the plugin is loaded at startup. True by default - the user can change it from the Plugin settings.
+*/
+bool PluginSpec::isEnabled() const
+{
+    return d->enabled;
+}
+
+/*!
+    \fn bool PluginSpec::isDisabledIndirectly() const
+    Returns true if loading was not done due to user unselecting this plugin or its dependencies,
+    or if command-line parameter -noload was used.
+*/
+bool PluginSpec::isDisabledIndirectly() const
+{
+    return d->disabledIndirectly;
+}
+
 /*!
     \fn QList<PluginDependency> PluginSpec::dependencies() const
     The plugin dependencies. This is valid after the PluginSpec::Read state is reached.
@@ -352,7 +421,7 @@ IPlugin *PluginSpec::plugin() const
 
     \sa PluginSpec::dependencies()
 */
-QList<PluginSpec *> PluginSpec::dependencySpecs() const
+QHash<PluginDependency, PluginSpec *> PluginSpec::dependencySpecs() const
 {
     return d->dependencySpecs;
 }
@@ -364,15 +433,20 @@ namespace {
     const char * const PLUGIN_NAME = "name";
     const char * const PLUGIN_VERSION = "version";
     const char * const PLUGIN_COMPATVERSION = "compatVersion";
+    const char * const PLUGIN_EXPERIMENTAL = "experimental";
     const char * const VENDOR = "vendor";
     const char * const COPYRIGHT = "copyright";
     const char * const LICENSE = "license";
     const char * const DESCRIPTION = "description";
     const char * const URL = "url";
+    const char * const CATEGORY = "category";
     const char * const DEPENDENCYLIST = "dependencyList";
     const char * const DEPENDENCY = "dependency";
     const char * const DEPENDENCY_NAME = "name";
     const char * const DEPENDENCY_VERSION = "version";
+    const char * const DEPENDENCY_TYPE = "type";
+    const char * const DEPENDENCY_TYPE_SOFT = "optional";
+    const char * const DEPENDENCY_TYPE_HARD = "required";
     const char * const ARGUMENTLIST = "argumentList";
     const char * const ARGUMENT = "argument";
     const char * const ARGUMENT_NAME = "name";
@@ -383,7 +457,10 @@ namespace {
     \internal
 */
 PluginSpecPrivate::PluginSpecPrivate(PluginSpec *spec)
-    : plugin(0),
+    :
+    enabled(true),
+    disabledIndirectly(false),
+    plugin(0),
     state(PluginSpec::Invalid),
     hasError(false),
     q(spec)
@@ -404,6 +481,7 @@ bool PluginSpecPrivate::read(const QString &fileName)
         = license
         = description
         = url
+        = category
         = location
         = "";
     state = PluginSpec::Invalid;
@@ -439,6 +517,16 @@ bool PluginSpecPrivate::read(const QString &fileName)
     return true;
 }
 
+void PluginSpec::setEnabled(bool value)
+{
+    d->enabled = value;
+}
+
+void PluginSpec::setDisabledIndirectly(bool value)
+{
+    d->disabledIndirectly = value;
+}
+
 /*!
     \fn bool PluginSpecPrivate::reportError(const QString &err)
     \internal
@@ -507,6 +595,14 @@ void PluginSpecPrivate::readPluginSpec(QXmlStreamReader &reader)
     } else if (compatVersion.isEmpty()) {
         compatVersion = version;
     }
+    QString experimentalString = reader.attributes().value(PLUGIN_EXPERIMENTAL).toString();
+    experimental = (experimentalString.compare(QLatin1String("true"), Qt::CaseInsensitive) == 0);
+    if (!experimentalString.isEmpty() && !experimental
+            && experimentalString.compare(QLatin1String("false"), Qt::CaseInsensitive) != 0) {
+        reader.raiseError(msgInvalidFormat(PLUGIN_EXPERIMENTAL));
+        return;
+    }
+    enabled = !experimental;
     while (!reader.atEnd()) {
         reader.readNext();
         switch (reader.tokenType()) {
@@ -522,6 +618,8 @@ void PluginSpecPrivate::readPluginSpec(QXmlStreamReader &reader)
                 description = reader.readElementText().trimmed();
             else if (element == URL)
                 url = reader.readElementText().trimmed();
+            else if (element == CATEGORY)
+                category = reader.readElementText().trimmed();
             else if (element == DEPENDENCYLIST)
                 readDependencies(reader);
             else if (element == ARGUMENTLIST)
@@ -646,6 +744,18 @@ void PluginSpecPrivate::readDependencyEntry(QXmlStreamReader &reader)
         reader.raiseError(msgInvalidFormat(DEPENDENCY_VERSION));
         return;
     }
+    dep.type = PluginDependency::Required;
+    if (reader.attributes().hasAttribute(DEPENDENCY_TYPE)) {
+        QString typeValue = reader.attributes().value(DEPENDENCY_TYPE).toString();
+        if (typeValue == QLatin1String(DEPENDENCY_TYPE_HARD)) {
+            dep.type = PluginDependency::Required;
+        } else if (typeValue == QLatin1String(DEPENDENCY_TYPE_SOFT)) {
+            dep.type = PluginDependency::Optional;
+        } else {
+            reader.raiseError(msgInvalidFormat(DEPENDENCY_TYPE));
+            return;
+        }
+    }
     dependencies.append(dep);
     reader.readNext();
     if (reader.tokenType() != QXmlStreamReader::EndElement)
@@ -721,9 +831,10 @@ bool PluginSpecPrivate::resolveDependencies(const QList<PluginSpec *> &specs)
         hasError = true;
         return false;
     }
-    QList<PluginSpec *> resolvedDependencies;
+    QHash<PluginDependency, PluginSpec *> resolvedDependencies;
     foreach (const PluginDependency &dependency, dependencies) {
         PluginSpec *found = 0;
+
         foreach (PluginSpec *spec, specs) {
             if (spec->provides(dependency.name, dependency.version)) {
                 found = spec;
@@ -731,22 +842,48 @@ bool PluginSpecPrivate::resolveDependencies(const QList<PluginSpec *> &specs)
             }
         }
         if (!found) {
-            hasError = true;
-            if (!errorString.isEmpty())
-                errorString.append("\n");
-            errorString.append(QCoreApplication::translate("PluginSpec", "Could not resolve dependency '%1(%2)'")
-                .arg(dependency.name).arg(dependency.version));
+            if (dependency.type == PluginDependency::Required) {
+                hasError = true;
+                if (!errorString.isEmpty())
+                    errorString.append(QLatin1Char('\n'));
+                errorString.append(QCoreApplication::translate("PluginSpec", "Could not resolve dependency '%1(%2)'")
+                    .arg(dependency.name).arg(dependency.version));
+            }
             continue;
         }
-        resolvedDependencies.append(found);
+        resolvedDependencies.insert(dependency, found);
     }
     if (hasError)
         return false;
+
     dependencySpecs = resolvedDependencies;
+
     state = PluginSpec::Resolved;
+
     return true;
 }
 
+void PluginSpecPrivate::disableIndirectlyIfDependencyDisabled()
+{
+    if (!enabled)
+        return;
+
+    if (disabledIndirectly)
+        return;
+
+    QHashIterator<PluginDependency, PluginSpec *> it(dependencySpecs);
+    while (it.hasNext()) {
+        it.next();
+        if (it.key().type == PluginDependency::Optional)
+            continue;
+        PluginSpec *dependencySpec = it.value();
+        if (dependencySpec->isDisabledIndirectly() || !dependencySpec->isEnabled()) {
+            disabledIndirectly = true;
+            break;
+        }
+    }
+}
+
 /*!
     \fn bool PluginSpecPrivate::loadLibrary()
     \internal
@@ -787,7 +924,8 @@ bool PluginSpecPrivate::loadLibrary()
     PluginLoader loader(libName);
     if (!loader.load()) {
         hasError = true;
-        errorString = libName + QString::fromLatin1(": ") + loader.errorString();
+        errorString = QDir::toNativeSeparators(libName)
+            + QString::fromLatin1(": ") + loader.errorString();
         return false;
     }
     IPlugin *pluginObject = qobject_cast<IPlugin*>(loader.instance());
@@ -862,12 +1000,12 @@ bool PluginSpecPrivate::initializeExtensions()
     \fn bool PluginSpecPrivate::stop()
     \internal
 */
-void PluginSpecPrivate::stop()
+IPlugin::ShutdownFlag PluginSpecPrivate::stop()
 {
     if (!plugin)
-        return;
-    plugin->shutdown();
+        return IPlugin::SynchronousShutdown;
     state = PluginSpec::Stopped;
+    return plugin->aboutToShutdown();
 }
 
 /*!
@@ -882,4 +1020,3 @@ void PluginSpecPrivate::kill()
     plugin = 0;
     state = PluginSpec::Deleted;
 }
-
