@@ -2,39 +2,43 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
 **
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Contact: Nokia Corporation (info@qt.nokia.com)
 **
-** Commercial Usage
-**
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
 **
 ** GNU Lesser General Public License Usage
 **
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this file.
+** Please review the following information to ensure the GNU Lesser General
+** Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** If you are unsure which license is appropriate for your use, please
-** contact the sales department at http://qt.nokia.com/contact.
+** In addition, as a special exception, Nokia gives you certain additional
+** rights. These rights are described in the Nokia Qt LGPL Exception
+** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+**
+** Other Usage
+**
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+** If you have questions regarding the use of this file, please contact
+** Nokia at qt-info@nokia.com.
 **
 **************************************************************************/
 
 #ifndef MAINWINDOW_H
 #define MAINWINDOW_H
 
-#include "core_global.h"
-
 #include "eventfilteringmainwindow.h"
+#include "icontext.h"
+#include "icore.h"
 
 #include <QtCore/QMap>
+#include <QtGui/QColor>
 
 QT_BEGIN_NAMESPACE
 class QSettings;
@@ -46,24 +50,23 @@ QT_END_NAMESPACE
 namespace Core {
 
 class ActionManager;
-class BaseMode;
-class BaseView;
+class StatusBarWidget;
 class EditorManager;
+class ExternalToolManager;
 class FileManager;
-class IContext;
+class HelpManager;
 class IWizard;
 class MessageManager;
 class MimeDatabase;
 class ModeManager;
 class ProgressManager;
+class NavigationWidget;
 class RightPaneWidget;
 class ScriptManager;
 class SettingsDatabase;
 class UniqueIDManager;
 class VariableManager;
-class VCSManager;
-class ViewManagerInterface;
-class IMode;
+class VcsManager;
 
 namespace Internal {
 
@@ -71,13 +74,15 @@ class ActionManagerPrivate;
 class CoreImpl;
 class FancyTabWidget;
 class GeneralSettings;
-class NavigationWidget;
 class ProgressManagerPrivate;
 class ShortcutSettings;
-class ViewManager;
+class ToolSettings;
+class MimeTypeSettings;
+class StatusBarManager;
 class VersionDialog;
+class SystemEditor;
 
-class CORE_EXPORT MainWindow : public EventFilteringMainWindow
+class MainWindow : public EventFilteringMainWindow
 {
     Q_OBJECT
 
@@ -87,14 +92,14 @@ public:
 
     bool init(QString *errorMessage);
     void extensionsInitialized();
-    void shutdown();
+    void aboutToShutdown();
 
     IContext *contextObject(QWidget *widget);
     void addContextObject(IContext *contex);
     void removeContextObject(IContext *contex);
     void resetContext();
 
-    void openFiles(const QStringList &fileNames);
+    void openFiles(const QStringList &fileNames, ICore::OpenFilesFlags flags);
 
     Core::ActionManager *actionManager() const;
     Core::FileManager *fileManager() const;
@@ -106,21 +111,22 @@ public:
     Core::VariableManager *variableManager() const;
     Core::ModeManager *modeManager() const;
     Core::MimeDatabase *mimeDatabase() const;
+    Core::HelpManager *helpManager() const;
 
-    VCSManager *vcsManager() const;
-    inline QSettings *settings() const { return m_settings; }
+    VcsManager *vcsManager() const;
+    QSettings *settings(QSettings::Scope scope) const;
     inline SettingsDatabase *settingsDatabase() const { return m_settingsDatabase; }
     virtual QPrinter *printer() const;
     IContext * currentContextObject() const;
     QStatusBar *statusBar() const;
-    void addAdditionalContext(int context);
-    void removeAdditionalContext(int context);
+
+    void updateAdditionalContexts(const Context &remove, const Context &add);
     bool hasContext(int context) const;
 
-    void updateContext();
-
     void setSuppressNavigationWidget(bool suppress);
 
+    void setOverrideColor(const QColor &color);
+
 signals:
     void windowActivated();
 
@@ -130,9 +136,9 @@ public slots:
     void exit();
     void setFullScreen(bool on);
 
-    QStringList showNewItemDialog(const QString &title,
-                                  const QList<IWizard *> &wizards,
-                                  const QString &defaultLocation = QString());
+    void showNewItemDialog(const QString &title,
+                           const QList<IWizard *> &wizards,
+                           const QString &defaultLocation = QString());
 
     bool showOptionsDialog(const QString &category = QString(),
                            const QString &page = QString(),
@@ -161,10 +167,12 @@ private slots:
     void updateFocusWidget(QWidget *old, QWidget *now);
     void setSidebarVisible(bool visible);
     void destroyVersionDialog();
-    void modeChanged(Core::IMode *mode);
+    void openDelayedFiles();
 
 private:
     void updateContextObject(IContext *context);
+    void updateContext();
+
     void registerDefaultContainers();
     void registerDefaultActions();
 
@@ -173,35 +181,39 @@ private:
 
     CoreImpl *m_coreImpl;
     UniqueIDManager *m_uniqueIDManager;
-    QList<int> m_globalContext;
-    QList<int> m_additionalContexts;
+    Context m_additionalContexts;
     QSettings *m_settings;
+    QSettings *m_globalSettings;
     SettingsDatabase *m_settingsDatabase;
     mutable QPrinter *m_printer;
     ActionManagerPrivate *m_actionManager;
     EditorManager *m_editorManager;
+    ExternalToolManager *m_externalToolManager;
     FileManager *m_fileManager;
     MessageManager *m_messageManager;
     ProgressManagerPrivate *m_progressManager;
     ScriptManager *m_scriptManager;
-    VariableManager *m_variableManager;
-    VCSManager *m_vcsManager;
-    ViewManager *m_viewManager;
+    QScopedPointer<VariableManager> m_variableManager;
+    VcsManager *m_vcsManager;
+    StatusBarManager *m_statusBarManager;
     ModeManager *m_modeManager;
     MimeDatabase *m_mimeDatabase;
+    HelpManager *m_helpManager;
     FancyTabWidget *m_modeStack;
     NavigationWidget *m_navigationWidget;
     RightPaneWidget *m_rightPaneWidget;
-    Core::BaseView *m_outputView;
+    Core::StatusBarWidget *m_outputView;
     VersionDialog *m_versionDialog;
 
-    IContext * m_activeContext;
+    IContext *m_activeContext;
 
     QMap<QWidget *, IContext *> m_contextWidgets;
 
-    BaseMode *m_outputMode;
     GeneralSettings *m_generalSettings;
     ShortcutSettings *m_shortcutSettings;
+    ToolSettings *m_toolSettings;
+    MimeTypeSettings *m_mimeTypeSettings;
+    SystemEditor *m_systemEditor;
 
     // actions
     QShortcut *m_focusToEditor;
@@ -219,6 +231,9 @@ private:
 #endif
 
     QToolButton *m_toggleSideBarButton;
+    QColor m_overrideColor;
+
+    QStringList m_filesToOpenDelayed;
 };
 
 } // namespace Internal
