@@ -422,7 +422,6 @@ void GettingStartedWelcomePageWidget::slotOpenExample()
     }
 
     QString tryFile;
-    files << proFile;
     if (isQmlProject) {
         tryFile = proFileInfo.path() + '/' + "/main.qml";
         if(!QFile::exists(tryFile))
@@ -436,10 +435,17 @@ void GettingStartedWelcomePageWidget::slotOpenExample()
         tryFile = proFileInfo.path() + "/main.cpp";
         if(!QFile::exists(tryFile))
             tryFile = proFileInfo.path() + '/' + proFileInfo.baseName() + ".cpp";
+        files << tryFile;
+    }
+    if (ProjectExplorer::ProjectExplorerPlugin::instance()->openProject(proFile)) {
+        Core::ICore::instance()->openFiles(files);
+        if (!helpFile.isEmpty()) {
+            // queue this to make sure it gets executed after the editor widget
+            // has been drawn, so we know whether to show a split help or not
+            QMetaObject::invokeMethod(this, "slotOpenContextHelpPage",
+                                      Qt::QueuedConnection, Q_ARG(QString, helpFile));
+        }
     }
-    Core::ICore::instance()->openFiles(files, static_cast<Core::ICore::OpenFilesFlags>(Core::ICore::SwitchMode | Core::ICore::StopOnLoadFail));
-    if (!tryFile.isEmpty() && Core::EditorManager::instance()->hasEditor(tryFile) && !helpFile.isEmpty())
-        slotOpenContextHelpPage(helpFile);
 }
 
 void GettingStartedWelcomePageWidget::slotOpenHelpPage(const QString& url)
